' Routines to display the narrative information about an individual.
' Those routines are used for generating individual and family reports.
'
' HISTORY
' Aug-2005	GenoPro			Creation.
' Sep-2005	Ron			Amendment
' Mar-2005	Ron			Amendment to report half siblings of only child
' Apr-2005 - Jun-2006			Various!
' Jul-2006	Ron			Amend WriteHtmlFamily to remove float right div if no pictures (avoid IE bug)

'===========================================================
' Return Relative Pronoun (his/her/its) according to gender of individual in lowercase
Function PnR(i)
	PnR = Dic.LookupEx("PnR_", i.Gender.ID)
End Function

'===========================================================
' Return the Personal Pronoun (he/she/it) according to gender of individual, with or without Upper case first letter
Function PnP(i)
	PnP = Dic.LookupEx("PnP_", i.Gender.ID)
End Function

'===========================================================
' Return Objective Pronoun (him/her/its) according to gender of individual in lowercase
Function PnO(i)
	PnO = Dic.LookupEx("PnO_", i.Gender.ID)
End Function


' Determine if two objects, typically two individuals or an individual and a collection, are alive or deceased
Function ToBe2(obj1, obj2)
	Dim nStatistics1, nStatistics2
	nStatistics1 = Util.GetStatisticsForIndividuals(obj1)
	nStatistics2 = Util.GetStatisticsForIndividuals(obj2)
	If (nStatistics1 < 4 AND nStatistics2 < 4) Then
		' single or multiple individuals alive (is/are)
		ToBe2=StrVerb("ToBe", True, nStatistics1 <= 1 AND nStatistics2 <= 1, "", "")
	Else
		' single or multiple individuals deceased (was/were)
		ToBe2=StrVerb("ToBe", False, nStatistics1 < 8 AND nStatistics2 < 8, "", "")
	End If
End Function

' Determine if an individual is alive or deceased
Function ToBe(i)
	ToBe = StrVerb("ToBe", i.IsDead = False, True, "", i.Gender.ID)
End Function

' Determine correct form of 'to be' verb for (grand)parents of an individual
' N.B. returns blank if one (grand)parent is alive and one is dead, or only one (grand)parent supplied

Function ToBe2a(father, mother)
	Dim nStatisticsI, nStatisticsF, nStatisticsM
	nStatisticsF = Util.GetStatisticsForIndividuals(father) And 5
	nStatisticsM = Util.GetStatisticsForIndividuals(mother) And 5
	Select Case nStatisticsF + nStatisticsM
	Case 10 				' both (grand)parents dead (were)
		ToBe2a = StrVerb("ToBe", False, False, "", "")	
	Case 2					' both (grand)parents alive (are)
		ToBe2a = StrVerb("ToBe", True, False, "", "")
	Case 1, 5, 6					'only one (grand)parent or one (grand)parent alive, the other dead
		ToBe2a = ""				' return blank
	End Select
End Function

' Return the keyword from an individual and a collection and/or another individual.
Function ToHave(i, c)
	Dim nStatistics, strGender
	strGender = i.Gender.ID
	nStatistics = Util.GetStatisticsForIndividuals(i)
	If (nStatistics >= 4) Then
		ToHave = StrVerb("ToHave", False, True, "", strGender)	 ' The individual deceased
		Exit Function
	End If
	' The individual is alive, so check if someone is dead in the collection
	nStatistics = Util.GetStatisticsForIndividuals(c)
	If (nStatistics >= 4) Then
		ToHave = StrVerb("ToHave", False, True, "", strGender)
	Else
		ToHave = StrVerb("ToHave", True, True, "", strGender)	' Everyone is alive
	End If
End Function

Function GetDicMFU(strKey, strGender)
	GetDicMFU=Util.IfElse(Dic.Peek(strKey & "_" & strGender) <> "",strKey & "_" & strGender , strKey)
End Function

Function StrDateSpan(oFrom, oTo)
	If Not Util.IsNothing(oFrom) Then
		If Not Util.IsNothing(oTo) Then		' from & to
			StrDateSpan = Trim(Util.FormatString(GetDateFormat("FromAndTo", ""), _
					StrSubstitute(StrDateToString(oFrom, "From"), g_RegEx_CDS), _
					StrSubstitute(StrDateToString(oTo, "To"), g_RegEx_CDS)))
		Else					' from only
			StrDateSpan = Trim(StrSubstitute(StrDateToString(oFrom, "Since"), g_RegEx_CDS))
		End If
	ElseIf Not Util.IsNothing(oTo) Then		' to only
		StrDateSpan = Trim(StrSubstitute(StrDateToString(oTo, "Until"), g_RegEx_CDS))
		StrDateSpan = ""
	End If
End Function

Function StrDateToString(oDate, strFormat)
	Dim strYMD, strFmt
	If oDate.Month = "" Then
		strYMD = "Y"
		StrDateToString = oDate.ToString(GetDateFormat(strFormat, strYMD))
	ElseIf oDate.Day = "" Then
		strYMD = "YM"
		' deal with 'd' bug in GenoPro date formatting by 1st replacing it with $ then changing $ back to 'd'
		strFmt = Replace(GetDateFormat(strFormat, strYMD),"", "")
		StrDateToString = Replace(oDate.ToString(strFmt),"","d")
	ElseIf oDate.Year = "" Then
		strYMD = "MD"
		StrDateToString = oDate.ToString(GetDateFormat(strFormat, strYMD))
	Else
		strYMD = "YMD"
		StrDateToString = oDate.ToString(GetDateFormat(strFormat, strYMD))
	End If
End Function

Function GetDateFormat(strFormat, strYMD)
	GetDateFormat = g_oDicCache.KeyValue("DR_" & strFormat & strYMD)
End Function

Function StrDicMFU(strKey, strGender)
	StrDicMFU=Dic.Lookup2(strKey & "_" & strGender, strKey)
End Function

' get attribute of Dictionary entry.
Function StrDicAttribute(strKey, strID)
	Dim oNode, strValue
	Set oNode = Nothing
	strValue = ""
	If Not g_oDicRepGen Is Nothing Then Set oNode = g_oDicRepGen.selectSingleNode(strKey)
	If not oNode Is Nothing Then
		strValue = oNode.GetAttribute(strID)
		If IsNull(strValue) Then strValue = ""
	Else
		Report.LogWarning Util.FormatString(Dic("WarningAttributeKeyMissing"), strKey)
	End If
	StrDicAttribute = strValue
End Function

' get attribute of Key1 if it exists, otherwise attribute of Key2
Function StrDicAttribute2(strKey1, strKey2, strID)
	If Dic.Peek(strKey1) <> "" Then
		StrDicAttribute2 = StrDicAttribute(strKey1, strID)
	Else
		StrDicAttribute2 = StrDicAttribute(strKey2, strID)
	End If
End Function

Function StrDicPeekAttribute(strKey, strID)
	If Dic.Peek(strKey) <> "" Then
		StrDicPeekAttribute = StrDicAttribute(strKey, strID)
	Else
		StrDicPeekAttribute = ""
	End If
End Function

Function StrNameTranslate(strName, oNameDic, fBlankIfNone)
	Dim strTemp, strTrans, strPart
	strTemp = strName
	If Not oNameDic Is Nothing And strTemp <> "" Then
		strTrans = oNameDic(strTemp)
		If strTrans <> strTemp Then
			StrNameTranslate = strTrans
			Exit Function
		End If
		Dim oMatches, oMatch
		g_RegEx.Pattern = "([\w-`'])+"
		g_RegEx.Global = True
		Set oMatches = g_RegEx.Execute(strTemp)
		For Each oMatch In oMatches
			strPart = oMatch.Value
			strTrans = oNameDic(strPart)
			If strTrans <> strPart Then strTemp = Replace(strTemp, strPart, strTrans)
		Next
	End If
	If Not fBlankIfNone Or strTemp <> strName Then
		 StrNameTranslate = strTemp
	Else
		StrNameTranslate = ""
	End If
End Function

Function StrPlaceTranslate(strName)
	Dim arrParts, strPart, strTrans, strTranslated, oNameDicPlace
	strTranslated = strName
	Set oNameDicPlace = Session("oNameDicPlace")
	If Not oNameDicPlace Is Nothing Then
		arrParts = split(strName, ",")
		For Each strPart In arrParts
			strTrans = oNameDicPlace(Trim(strPart))
			If strTrans <> strPart Then strTranslated = Replace(strTranslated, Trim(strPart), strTrans)
		Next
	End If
	StrPlaceTranslate = strTranslated			
End Function

Function StrSubstitute(strValue, arrPatterns)
	Dim i,strTemp
	For i = 0 to Ubound(arrPatterns)-1 Step 2
		g_RegEx.Pattern=arrPatterns(i)
		strTemp=g_RegEx.Replace(strValue, arrPatterns(i+1))
		If strValue <> strTemp Then Exit For
	Next
	If strTemp <> "" Then
		StrSubstitute = strTemp
	Else
		StrSubstitute = strValue
	End If
End Function


Function StrVerb(strRoot, fPresent, fSingular, strVariant, strGender)
	' return required form of verb i.e. present/past tense, singular or plural, language variant
	Dim strKey
	strKey = "_" & strRoot & strVariant & Util.IfElse(fPresent, "_Present", "_Past")
	If strGender <> "" Then If Dic.Peek(strKey & "_" & strGender)<> "" Then strKey = strKey & "_" & strGender
	StrVerb = Dic.Plurial(strKey, fSingular + 2)		'n.b. 'fSingluar + 2' equals 1 if fSingular is true and 2 (i.e. plural) if false.
End Function


Sub WriteNarrativeIndividual(i, fShortFormat, fam)

	Dim strGender, strName, strNamePossessive, strPnR, strPnP, strToHave, ich
	strGender = i.Gender.ID
	strName = Util.FormatPhrase(StrDicMFU("PhNameFriendly", strGender) ,i.Session("NameFormal"), i.Session("NameFull"), i.Session("NameKnownAs"))
	strNamePossessive = i.Session("NamePossessive")
	strPnR = PnR(i)
	strPnP = PnP(i)
	
	Dim b, ba, c, d, f
	Set b = i.Birth
	Set ba = b.Baptism
	Set d = i.Death
	Set f = d.Funerals
	Report.WritePhrase StrDicMFU("PhBirth",strGender), strName, b.Date.ToStringNarrative, b.Place.Session("HlinkLocative"), ba.Date.ToStringNarrative, _
				ba.Place.Session("HlinkLocative"), ba.Officiator.Title, StrFormatText(i, ba.Officiator), (b.Place.ID = ba.Place.ID), _
				StrFormatText(i, b.Doctor), b.PregnancyLength.Months,  b.PregnancyLength.Weeks, strPnR, b.CeremonyType, strNamePossessive
	WriteHtmlFootnoteRef2 b.Source, ba.Source
	g_RegEx.Pattern = "[,&]"
	Report.WritePhrase StrDicMFU("PhGodparents", strGender), strPnR, StrFormatText(i, ba.Godfather), StrFormatText(i, ba.Godmother), i.IsDead = False, g_RegEx.Test(ba.Godfather), g_RegEx.Test(ba.Godmother)

	If (Not fShortFormat) Then
		' Write the parents and grandparents
		Report.WritePhrase StrDicMFU("PhParents", strGender), strNamePossessive, ToBe(i.Father), i.Father.Session("HlinkNN"), _
							      strPnR, ToBe(i.Mother), i.Mother.Session("HlinkNN")
		Report.WritePhrase StrDicMFU("PhGrandParents", strGender), strPnR, ToBe2a(i.Father.Father, i.Father.Mother), _
									ToBe(i.Father.Father), i.Father.Father.Session("HlinkNN"), _
									ToBe(i.Father.Mother), i.Father.Mother.Session("HlinkNN"), _
									ToBe2a(i.Mother.Father, i.Mother.Mother), _
									ToBe(i.Mother.Father), i.Mother.Father.Session("HlinkNN"), _
									ToBe(i.Mother.Mother), i.Mother.Mother.Session("HlinkNN")
		WriteNarrativeSiblings i
	End If

	If (i.IsDead) Then
		Set c = d.Cause
		ich = Report.BufferLength
		Report.WritePhrase StrDicMFU("PhDied",strGender), strPnP, i.Death.Age.Years, i.Death.Age.Months, i.Death.Age.Days, d.Date.ToStringNarrative, d.Place.Session("HlinkLocative"), c, StrFormatText(i, c.Description), i.Birth.Date.Approximation & i.Death.Date.Approximation
        	Report.WritePhrase StrDicMFU("PhFuneral",strGender), f.Date.ToStringNarrative, f.Place.Session("HlinkLocative"), StrFormatText(i, f.Agency)
		Report.WritePhrase StrDicMFU("PhBurial",strGender) , strPnR, d.Disposition.type, d.Disposition.Date.ToStringNarrative, d.Disposition.Place.Session("HlinkLocative")
		If (ich = Report.BufferLength) And (Not i.Father.IsDead = True Or Not i.Mother.IsDead = True) Then Report.WritePhraseDic "PhDead", strName ' Indicate dead if no other death phrase and one or more parents alive.
	End If
	WriteHtmlFootnoteRef2 f.Source, d.Disposition.Source
	WriteHtmlFootnoteRef d.Source
	
	Dim collSpouses, collChildren, strHtmlChildren
	Set collSpouses = i.Mates.ToGenoCollection
	Set collChildren = i.Children.ToGenoCollection.SortByGender

	' if family report page remove current spouse and this family's children and report on other spouse & children
	If Not Util.IsNothing(fam) Then
		collSpouses.Remove i.FindMate(fam)
		collChildren.Remove fam.Children
		strHtmlChildren = StrHtmlCollectionOtherChildren(collChildren)
	ElseIf fShortFormat Or g_fHideFamilyDetails Then
		strHtmlChildren = StrHtmlCollectionChildren(collChildren)
	Else
		strHtmlChildren = StrHtmlCollectionChildrenLocal(collChildren, "")
	End If
	' {0=Name} {1=has|had} {n spouses} named, ..., and {n sons} and {n daughters}, named ...
	strToHave = StrVerb("ToHave", False, True, "", strGender)
	If (collSpouses.Count = 1) Then
		' One spouse, therefore check if they are still together
		If (i.Families(0).AreTogether) Then
			strToHave = StrVerb("ToHave", True, True, "", strGender)
		End If
	End If
	' The following lines have been commented because of linguistic issues such as "spouse", "husband", "wife" and "partner".
	' Until a solution is developed, those lines will remain commented by default.
	' if family report page only report other spouse(s) and children
	If Util.IsNothing(fam) Then
		'Report.WritePhraseDic "PhSpousesAndChildren", strName, strToHave, StrHtmlCollectionSpouses(collSpouses), strHtmlChildren
	Else
		'Report.WritePhraseDic "PhOtherSpousesAndChildren", strName, strToHave, StrHtmlCollectionOtherSpouses(collSpouses), strHtmlChildren
	End If
End Sub

Sub WriteNarrativeUnionsAndDivorces(f, nFamily, iParent)
	Dim strPrefix, strToBe, collParents, collUnions, m, d, strAlso
	strAlso = ""
	Set collParents = f.Parents.ToGenoCollection
	If nFamily >= 0 Then
		If iParent.ID = collParents(0).ID Then
			strPrefix = Dic.FormatPhrase("PhThey", collParents(0).Session("NameShort"), StrHtmlNarrativeName(collParents(1),"NameShort", "", ""))
		Else
			strPrefix = Dic.FormatPhrase("PhThey", collParents(1).Session("NameShort"), StrHtmlNarrativeName(collParents(0),"NameShort", "", ""))
		End If
	Else		
		strPrefix = Dic.FormatPhrase("PhThey", collParents(0).Session("NameShort"), StrHtmlNarrativeName(collParents(1),"NameShort", "", ""))
	End If
	strToBe = StrVerb("ToBe", f.AreTogether, False, "2", "")
	Set collUnions = f.Unions.ToGenoCollection
	If (collUnions.Count > 0) Then
		' The family has at least one union/marriage
		g_RegEx.Pattern = "[,&]"
		For Each m In collUnions
			Report.WritePhraseDic "PhUnion", strPrefix, m.Type, m.Date.ToStringNarrative, m.Place.Session("HlinkLocative"), m.Officiator.Title, m.Officiator, m.Witnesses, strToBe, strAlso, g_RegEx.Test(m.Witnesses)
			WriteHtmlFootnoteRef m.Source
			Set d = m.Divorce
			Report.WritePhraseDic "PhDivorce", m.IsAnnulled, d.Date.ToStringNarrative, d.Place.Session("HlinkLocative"), d.RequestedBy.ID, _
							 d.RequestedBy, StrFormatText(m, d.Attorney.Husband), StrFormatText(m, d.Attorney.Wife), StrFormatText(m, d.Officiator)
			WriteHtmlFootnoteRef m.Divorce.Source
			WriteHtmlAdditionalInformation(m)
			WriteHtmlAnnotation m, Dic("AnnotationUnion"), m.Comment
			strPrefix = Dic.Plurial("PnP_",2)
			strAlso = Dic("Also")
		Next
	Else
		' There is no union/marriage to the family, so display the relationship
		Dim strFmtRelation
		strFmtRelation = Dic("PhFR_" & f.Relation.ID)	' The dictionary contains formatting elements for the relation
		If Instr(f.Relation.ID,"Separation") > 0 Or Instr(f.Relation.ID,"Decease") > 0 Then
			strToBe = StrVerb("ToBe", False, False, "2", "")
		ElseIf Util.GetStatisticsForIndividuals(collParents) < 4 Then
			strToBe = StrVerb("ToBe", True, False, "2", "")
		Else
			strToBe = StrVerb("ToBe", False, False, "2", "")
		End If
		Report.WritePhrase strFmtRelation, strPrefix, strToBe, (collParents(0).Name <> "" And collParents(1).Name <> "")
	End If
End Sub


' Write details of a family
'
' This routine has two extra parameters (nFamily and iParent) to
' optionally add the spouse name to the report.
' If nFamily is zero, then don't add the spouse name because there is only one spouse (it's redundant)
' If nFamily is negative, it is because we are generating the HTML code for a family report rather than an individual's family
Sub WriteHtmlFamily(f, nFamily, iParent)
	Dim collChildren, collAdopted, collFoster, oRepertoryNonBio, c, spouse, strSpouseName, strSpouseType, strHaveOrHad
	Dim strHaveOrHad1, strHyperlinkText, strName, strPrefix, ich, strAlso
	Dim strOrdinalGender, strOrdinal
	If (nFamily >= 0) Then
		Set spouse = iParent.FindMate(f)	' Find the other spouse
		strSpouseName = spouse.Session("NameFull")
		If (strSpouseName = "") Then
			strSpouseName = StrDicMFU("_NoName", spouse.Gender.ID)
		End If
		strPrefix = Util.IfElse(f.GotMarried, "_Spouse", "_Partner")
		strSpousePossessive = StrDicAttribute2(strPrefix & "_" & iParent.Gender.ID & "_" & spouse.Gender.ID, strPrefix,"G") ' get genetive case of spouse noun
		strOrdinalGender = StrDicAttribute("PhFamilyWith", "G1")	' optional G1 attribute indicates gender  required for parameter {1} (ie. first, second, third etc) in other language translations
		If nFamily > 0 Then strOrdinal = StrDicMFU("_Ordinal_" & nFamily, strOrdinalGender)
		strHyperlinkText = Dic.FormatPhrase("PhFamilyWith", i.Session("NamePossessive"), strOrdinal, strSpouseName, "")
		Report.WriteFormattedLn "<div class='clear'><br /><{0}>{1}</{0}></div>", Util.IfElse(g_fHideFamilyDetails, "span", "h2"), Replace(Util.FormatHtmlHyperlink(f.Href, strHyperlinkText),">"," onclick='javascript:hidePopUpFrame();' >",1,1)
	End If

'	put pictures in a 'div' floating right. If no pictures added then remove the 'div' 

	Dim cchBufferStart	' Start of the write operations
	Dim cchBufferBegin	' The begining of the picture data

	cchBufferStart = Report.BufferLength

	Report.WriteLn "<div class='floatright aligncenter widthpaddedlarge'>"

	cchBufferBegin = Report.BufferLength

	If (Not Util.IsNothing(spouse)) Then
		WriteHtmlPicturesLarge spouse, "", Util.StrFirstCharUCase(Dic.FormatString("FmtPictures", strSpousePossessive)), ""
		If g_fShowPictureDetails Then
			WriteHtmlDetailsPicture spouse, "", False
			WriteHtmlAnnotationPicture strSpouseType, spouse, "right", g_cxPictureSizeLarge
		End If
		If f.Pictures.Count > 0 Then Report.WriteBr
		WriteHtmlPicturesLarge f, "", Dic.FormatString("FmtPictures", Dic("Family")), g_fHidePictureDetails
        Else
		WriteHtmlPicturesLarge f, "", "", g_fHidePictureDetails
	End If
	If g_fShowPictureDetails Then
		WriteHtmlDetailsPicture f, "", False
		WriteHtmlAnnotationPicture Dic("Family"), f, "", g_cxPictureSizeLarge
	End If

	If Report.BufferLength = cchBufferBegin Then		' no picture information written
		Report.BufferLength = cchBufferStart		' so remove the 'div' by stepping back the buffer
	Else
		Report.WriteLn "</div>"				' close the 'div' with end tag
	End If

	Report.WriteLn "<div>"
	WriteNarrativeUnionsAndDivorces f, nFamily, iParent

	' They {have|had} {n sons} and {n daughters}, named ...

	Set collChildren = f.Children.ToGenoCollection.SortByGender
	Set collAdopted = Util.NewGenoCollection
	Set collAdoptedByFather = Util.NewGenoCollection
	Set collAdoptedByMother = Util.NewGenoCollection
	Set collFoster = Util.NewGenoCollection

	Set oRepertoryNonBio = Session("oRepertoryNonBio")

	Dim strType
	strType = ""

	If oRepertoryNonBio.KeyCounter("A" & f.ID) > 0 Then
		Set oEntry = oRepertoryNonBio.Entry("A" & f.ID)
		For cnt = 0 To oEntry.Count-1 Step 2
			Set oLink = oEntry.Object(cnt)
			If oLink.Child.Mother.ID = f.Parents(1).ID Then
				collAdoptedByFather.Add oEntry.Object(cnt+1)
			ElseIf oLink.Child.Father.ID = f.Parents(0).ID Then
				collAdoptedByMother.Add oEntry.Object(cnt+1)
			Else
				collAdopted.Add oEntry.Object(cnt+1)
			End If
		Next
		collChildren.Remove collAdopted
		collChildren.Remove collAdoptedByFather
		collChildren.Remove collAdoptedByMother
		collAdopted.SortByGender
		collAdoptedByFather.SortByGender
		collAdoptedByMother.SortByGender
		strType="Biological"
	End If

	If oRepertoryNonBio.KeyCounter("F" & f.ID) > 0 Then	
		Set oEntry = oRepertoryNonBio.Entry("F" & f.ID)
		For cnt = 0 To oEntry.Count-1 Step 2
			collFoster.Add oEntry.Object(cnt+1)
		Next
		collChildren.Remove collFoster
		collFoster.SortByGender
		strType="Biological"
	End If

	strHaveOrHad1 = StrVerb("ToHave", Util.GetStatisticsForIndividuals(iParent, spouse) < 4, False, "", "")

	If (Util.GetStatisticsForIndividuals(collChildren) >= 4) Then
		strHaveOrHad = StrVerb("ToHave", False, False, "", "")	' One or more children died
	Else
		strHaveOrHad = strHaveOrHad1
	End If
	strPrefix = Dic.Plurial("PnP_", 2)	'they
	strAlso = ""
	ich = Report.BufferLength
	Report.WritePhraseDic "PhSpousesAndChildren", strPrefix, strHaveOrHad, StrHtmlCollectionChildrenLocal(collChildren, strType), nothing, strAlso
	If ich <> Report.BufferLength Then
		strPrefix = Dic.Plurial("PnP_",2)
		strAlso = Dic("Also")
	End If
	Report.WritePhraseDic "PhStepChildrenAdoption", f.Parents(0).Session("NameShort"), PnR(f.Parents(0)), StrHtmlCollectionStepChildren(collAdoptedByFather)
	Report.WritePhraseDic "PhStepChildrenAdoption", f.Parents(0).Session("NameShort"), PnR(f.Parents(1)), StrHtmlCollectionStepChildren(collAdoptedByMother)

	If (Util.GetStatisticsForIndividuals(collAdopted) >= 4) Then
		strHaveOrHad = StrVerb("ToHave", False, False, "", "")	' One or more adopted children died
	Else
		strHaveOrHad = strHaveOrHad1
	End If
	ich = Report.BufferLength
	Report.WritePhraseDic "PhSpousesAndChildren", strPrefix, strHaveOrHad, StrHtmlCollectionChildrenLocal(collAdopted, "Adopted"), nothing, strAlso
	If ich <> Report.BufferLength Then
		strPrefix = Dic.Plurial("PnP_",2)
		strAlso = Dic("Also")
	End If
	If (Util.GetStatisticsForIndividuals(collFoster) >= 4) Then
		strHaveOrHad = StrVerb("ToHave", False, False, "", "")	' One or more adopted children died
	Else
		strHaveOrHad = strHaveOrHad1
	End If
	Report.WritePhraseDic "PhSpousesAndChildren", strPrefix, strHaveOrHad, StrHtmlCollectionChildrenLocal(collFoster, "Foster"), nothing, strAlso
	Report.WritePhraseDic "PhFL_" & f.FamilyLine.ID, strHaveOrHad, f.AreTogether AND NOT f.EndOfFamily, f.Children.ToGenoCollection.Count > 0, ""

	WriteNarrativeTwins f	' put here because may include notes on each multiple birth set.

	WriteHtmlAnnotation f, Dic("AnnotationFamily"), f.Comment
	Report.WriteBr "</div>"

	WriteHtmlExtraNarrative f

	If (nFamily < 0) Then
                ' add short details of each partner and any other partner and/or children
		Set collParents = f.Parents.ToGenoCollection
		Report.WriteBr	"<div class='clearleft'><ul class='xT'>"
		Report.WriteFormattedLn "    <li class='xT2-{} xT-h' onclick='xTclk(event,""2"")'>", Util.IfElse(g_fCollapseReferences, "c", "o")
		Report.WriteFormattedLn "<a name='Personal'></a><h3 class='xT-i inline'>{&t}</h3><ul class='xT-h'><li>", Dic("HeaderPersonal")
		For Each p in collParents
			WriteHtmlIndividual p, f
		Next
		Report.WriteLn "</li></ul></li></ul></div>"
	End If

	Set collChildren = f.Children.ToGenoCollection
	If (collChildren.Count > 0) Then
		g_ichReferencesStart = -1 ' indicate at least one collapse/expand non-notes section present
		Report.WriteBr	"<div class='clearleft'><ul class='xT'>"
		Report.WriteFormattedLn "    <li class='xT2-{} xT-h' id='Children' onclick='xTclk(event,""2"")'>", Util.IfElse(g_fCollapseReferences, "c", "o")
		Report.WriteFormattedLn "<a name='Children'></a><h3 class='xT-i inline'>{&t}</h3><ul class='xT-h'><li>", Dic("HeaderChildren")
		Report.WriteLn "<div class='childrenindent'>"
		For Each c In collChildren
			Report.WriteFormattedLn "<a name='{&t}'/>", c.Href
			WriteHtmlIndividual c, Null
		Next  
		Report.WriteLn "</div>"
		Report.WriteLn "</li></ul></li></ul></div>"
	End If


	If (nFamily >= 0) Then
		WriteHtmlAdditionalInformation(f)
	End If

End Sub

Sub WriteHtmlIndividual(i, f)
	If isPrivate(i) And Left(Report.FileOutput,g_nPrivateFolder)<>g_strPrivateFolder Then
		Report.WriteFormattedLn "<div class='clearleft'>{} {}<br/></div>", StrHtmlImgGender(i), StrHtmlHyperlinkNN(i)
	Else
	Report.WriteLn "<div class='clearleft'>"
	Report.Write StrHtmlImgGender(i)
	Report.WriteBr i.Session("HlinkNN")
	Set p = i.Pictures.Primary
	If (Not Util.IsNothing(p)) Then
		Report.WriteLn "<div class='floatleft aligncenter widthpaddedsmall'>"
		WriteHtmlPictureSmall p, ""
		WriteHtmlNamePicture i, p
		Report.WriteLn "</div>"
	End If
	WriteNarrativeIndividual i, true, f
	Report.WriteBr "</div>"
	End If
End Sub

Sub WriteNarrativeSiblings(i)
	Dim collSiblings, cSiblings, strPrefix, collAdoptedSiblings, collFosterSiblings, collOtherSiblings
	Dim iSibling, oEntry, oLink, c, f, fEndOfLine, fOrdered, oRepertoryTwins, strGender, cMales, cFemales
	strGender = i.Gender.ID
	cMales = 0
	cFemales = 0
	If strGender = "M" Then
		cMales = 1
	ElseIf strGender = "F" Then
		cFemales = 1
	End If
	Set oRepertoryTwins = Session("oRepertoryTwins")
	Set oRepertoryNonBio = Session("oRepertoryNonBio")
	Set collSiblings = Util.NewGenoCollection
	Set collOtherSiblings = Util.NewGenoCollection
	Set collHalfSiblings = Util.NewGenoCollection
	Set f = i.Family
	For Each iSibling In i.Siblings.ToGenoCollection
		If f.ID = iSibling.Family.ID Or oRepertoryNonBio.KeyCounter("I"&iSibling.ID) = 0 Then
			collSiblings.Add iSibling
		Else
			collOtherSiblings.Add iSibling
		End If
		If iSibling.Gender.ID = "M" Then
			cMales = cMales + 1
		ElseIf iSibling.Gender.ID = "F" Then
			cMales = cFemales + 1
		End If
	Next
	collHalfSiblings.Add i.Siblings.Half.ToGenoCollection
	Set collOtherSiblings = i.Siblings.other.ToGenoCollection
	cSiblings = i.Siblings.ToGenoCollection.Count
	If (cSiblings = 0) Then
		If (Not Util.IsNothing(i.Family)) Then
			If collHalfSiblings.Count > 0 Then
				WriteNarrativeMFU i, collHalfSiblings, "SiblingHalf"
			Else
				If f.FamilyLine.ID ="NoMoreChildren" Or f.FamilyLine.ID = "" Then Report.WritePhrase StrDicMFU("PhOnlyChild",strGender), PnP(i), ToBe(i)
			End If
		End If
		WriteNarrativeAdoption i
		Exit Sub
	End If
	Dim strChildRank, strChildRank1, nChildRank, cChildren
	cChildren = cSiblings + 1
	nChildRank = i.FamilyRank
	strChildRank1 = ""
	If (nChildRank <= 1) Then
		If cChildren = 2 Then
			strChildRank = Dic.PlurialCardinal(GetDicMFU("Oldest", strGender), cChildren)'
		Else
			strChildRank = Dic.Plurial(GetDicMFU("Oldest", strGender), cChildren)
		End If
	ElseIf (nChildRank = cChildren) Then
		If cChildren = 2 Then
			strChildRank = Dic.PlurialCardinal(GetDicMFU("Youngest", strGender), cChildren)
		Else
			strChildRank = Dic.Plurial(GetDicMFU("Youngest", strGender), cChildren)
		End If
	Else
		strChildRank = StrDicMFU("_Ordinal_" & nChildRank, strGender)
		strChildRank1 = Dic("Oldest")
		fOrdinal = True
	End If
	strPrefix = Pnp(i)
	If Util.IsNothing(i.Family) Then ' individual is adopted or fostered but birth parent(s) not present
		Dim oRepertoryNonBio
		If oRepertoryNonBio.KeyCounter("I" & i.ID) > 0 Then
			Set oLink = oRepertoryNonBio.Entry("I" & i.ID).Object(0)
			strPrefix = Util.FormatPhrase(Dic("PhPL_" & oLink.PedigreeLink.ID & "3"),PnR(i), PnP(i))
			Set f = oLink.Family
		End If
	End If

	WriteNarrativeMFU i, collSiblings, "Sibling"
	Dim collAdoptedByFather, collAdoptedByMother
	Set collAdoptedByFather = Util.NewGenoCollection
	Set collAdoptedByMother = Util.NewGenoCollection
	If oRepertoryNonBio.KeyCounter("A" & i.Family.ID) > 0 Then
		collSiblings.Clear
		Set oEntry = oRepertoryNonBio.Entry("A" & i.Family.ID)
		For cnt = 0 To oEntry.Count-1 Step 2
			Set oLink = oEntry.Object(cnt)
			If oLink.Child.Mother.ID = i.Mother.ID Then
				collHalfSiblings.Add oEntry.Object(cnt+1)
				collAdoptedByFather.Add oEntry.Object(cnt+1)
			ElseIf oLink.Child.Father.ID = i.Father.ID Then
				collHalfSiblings.Add oEntry.Object(cnt+1)
				collAdoptedByMother.Add oEntry.Object(cnt+1)
			Else
				collSiblings.Add oEntry.Object(cnt+1)
			End If
		Next
		WriteNarrativeMFU i, collSiblings, "SiblingAdopted"
	End If

	If oRepertoryNonBio.KeyCounter("F" & i.Family.ID) > 0 Then	
		collSiblings.Clear
		Set oEntry = oRepertoryNonBio.Entry("F" & i.Family.ID)
		For cnt = 0 To oEntry.Count-1 Step 2
			collSiblings.Add oEntry.Object(cnt+1)
		Next
		WriteNarrativeMFU i, collSiblings, "SiblingFoster"
	End If

	fOrdered = True
	If (Not Util.IsNothing(f)) Then
	If f.Children.OrderUnknown Then
		fOrdered = False
	ElseIf f.Children.Order.ToGenoCollection.Count = 0 Then
		If  i.Birth.Date.ToStringNarrative = "" Then
			fOrdered = False
		Else
			For Each c in i.Siblings.All.ToGenoCollection
				If c.Birth.Date.ToStringNarrative = "" Then
					fOrdered = False
					Exit For
				End If
			Next
		End If
	End If
	fEndOfLine = (f.FamilyLine.ID = "" Or f.FamilyLine.ID = "NoMoreChildren")
	End If

	If fOrdered And oRepertoryNonBio.KeyCounter("A" & f.ID) = 0 And oRepertoryNonBio.KeyCounter("F" & f.ID) = 0 And oRepertoryTwins.KeyCounter("F" & f.ID) = 0 Then
			Report.WritePhrase StrDicMFU("PhChildRank", strGender), strPrefix, ToBe(i), strChildRank, strChildRank1, Dic.Cardinal(cChildren), Util.IfElse(fEndOfLine,"", StrDicAttribute2("Known_" & strGender, "Known", "P")), Dic.Plurial("Child", cChildren)
	End If
	WriteNarrativeTwins i

	WriteNarrativeAdoption i

	WriteNarrativeMFU i, collHalfSiblings, "SiblingHalf"
	Report.WritePhraseDic "PhAdoptedBy", i.Father.Session("HlinkNN"), StrHtmlNarrativeNamesShort(collAdoptedByFather)
	Report.WritePhraseDic "PhAdoptedBy", i.Mother.Session("HlinkNN"), StrHtmlNarrativeNamesShort(collAdoptedByMother)

End Sub

' Write a narrative phrases, breaking the collection by males, females and unknown gender.
' This routine is used to write the siblings and half siblings.
' Example: "He had a half-brother and two half-sisters, named Benoit, Anne and Estelle."
Sub WriteNarrativeMFU(i, coll, strDicPrefix)
	coll.SortByGender
	Report.WritePhrase StrDicMFU("PhMalesFemalesUnknowns", i.Gender.ID), PnP(i), ToHave(i, coll), StrHtmlCollectionMFU(coll, strDicPrefix, StrHtmlNarrativeNamesShort(coll))
End Sub


Sub WriteNarrativeAdoption(obj)
	Dim oRepertoryNonBio, oLink, oLink1, i,j,f,idFamily,idFamilyBirth, oAdopt, coll, strAlso, strAdopter, strWho, strGender
	Set oRepertoryNonBio = Session("oRepertoryNonBio")
	Set coll = Util.NewGenoCollection
	If oRepertoryNonBio.KeyCounter("I" & obj.ID) > 0 Then
		strGender = obj.Gender.ID
		strAlso = ""
		For i = 0 to oRepertoryNonBio.Entry("I" & obj.ID).Count-1
			Set oLink = oRepertoryNonBio.Entry("I" & obj.ID).Object(i)
			idFamily =oLink.Family.ID
			idFamilyBirth = "B" & obj.Family.ID
			If oRepertoryNonBio.KeyCounter(idFamilyBirth) > 1 Then
				For j = 0 to oRepertoryNonBio.Entry(idFamilyBirth).Count-1
					Set oLink1 =  oRepertoryNonBio.Entry(idFamilyBirth).Object(j)
					If oLink1.individual.ID <> obj.ID And oLink1.Family.ID = idFamily Then 'not this individual and the same adopters
						coll.Add oLink1.individual
					End If
				Next
			End If
			strAdopter = oLink.Family.Session("Hlink")
			strWho=Dic.Plurial("PnP_",2)
			If oLink.Family.Parents(1) = obj.Mother  Then
				strAdopter=oLink.Family.Parents(0).Session("Hlink")
				strWho=PnP(oLink.Family.Parents(0))
			ElseIf oLink.Family.Parents(0) = obj.Father Then
				strAdopter=oLink.Family.Parents(1).Session("Hlink")
				strWho=PnP(oLink.Family.Parents(1))
			End If
			Set oAdopt = oLink.Adoption
			Report.WritePhrase StrDicMFU("PhPL_" & oLink.PedigreeLink.ID, strGender), PnP(obj), strAdopter, oAdopt.Age, oAdopt.Date.ToStringNarrative, oAdopt.Place.Session("HlinkLocative"), oAdopt.Agency, StrHtmlNarrativeNamesShort(coll), strAlso, strWho
			WriteHtmlFootnoteRef oAdopt.Source
			WriteHtmlAnnotation oLink, Dic("Annotation"+oLink.PedigreeLink.ID), oLink.Comment
			strAlso = Dic("Also")
		Next
	End If
End Sub

Sub WriteNarrativeTwins(obj)
	Dim oRepertoryTwins, strTwinId, oTwin, oTwins, i
	Set oRepertoryTwins = Session("oRepertoryTwins")

	Select Case obj.class
	Case "Family"
		If oRepertoryTwins.KeyCounter("F" & obj.ID) > 0 Then
			For i = 0 to oRepertoryTwins.Entry("F" & obj.ID).Count-1
				strTwinId = oRepertoryTwins.Entry("F" & obj.ID).Object(i)
				WriteNarrativeTwin oRepertoryTwins.Entry(strTwinId).Object(0)
			Next
		End If
	Case "Individual"				
		If oRepertoryTwins.KeyCounter("I" & obj.ID) > 0 Then
			strTwinId = oRepertoryTwins.Entry("I" & obj.ID).Object(0)
			WriteNarrativeTwin oRepertoryTwins.Entry(strTwinId).Object(0)
		End If
	End Select
End Sub

Sub WriteNarrativeTwin(oTwinLink)
	Dim oTwin, oTwins, fSomeAlive, cMales, cFemales, strGender
	set oTwins = oTwinLink.Siblings.ToGenoCollection
	fSomeAlive = True
	cMales = 0
	cFemales = 0
	For Each oTwin in oTwins
		fSomeAlive = fSomeAlive And Not oTwin.IsDead
		If oTwin.Gender.ID = "M" Then cMales = cMales + 1
		If oTwin.Gender.ID = "F" Then cFemales = cFemales + 1
	Next
	If cMales = oTwins.Count then strGender = "M"
	If cFemales = oTwins.Count then strGender = "F"
	Report.WritePhraseDic "PhTwins", StrHtmlNarrativeNamesShort(oTwins), _
							fSomeAlive = True, Util.FirstNonEmpty(Dic.Peek(oTwinlink.TwinLink.ID & "_" & strGender), oTwinlink.TwinLink), Dic.PlurialCardinal(GetDicMFU("Twins", strGender), oTwins.Count)
	WriteHtmlAnnotation oTwinLink, Util.StrFirstCharUcase(Dic.PlurialCardinal("Twins", oTwins.Count)) & " " & Util.StrFirstCharUcase(Dic("Notes")),oTwinLink.Comment
	WriteHtmlFootnoteRefs oTwinLink.Sources
End Sub

' Return a string containing the collection summary by male, females and unknown gender, followed by the names in HTML format.
' Example: "a brother and two sisters, named Benoit, Anne and Estelle"
Function StrHtmlCollectionMFU(coll, strDicPrefix, strHtmlNames)
	Dim i, cMales, cFemales
	For Each i In coll
		Select Case i.Gender.ID
		Case "M"
			cMales = cMales + 1
		Case "F"
			cFemales = cFemales + 1
		End Select
	Next
	StrHtmlCollectionMFU = Dic.FormatPhrase("PhCollectionMFU", _
		Dic.PlurialCardinal(strDicPrefix &"_M", cMales), _
		Dic.PlurialCardinal(strDicPrefix & "_F", cFemales), _
		Dic.PlurialCardinal(strDicPrefix & "_", coll.Count - cMales - cFemales), strHtmlNames, coll.Count - cFemales = 0, coll.Count > 1)
End Function

Function StrHtmlCollectionSpouses(collSpouses)
	StrHtmlCollectionSpouses = StrHtmlCollectionMFU(collSpouses, "Spouse", StrHtmlNarrativeNamesFull(collSpouses))
End Function
Function StrHtmlCollectionOtherSpouses(collSpouses)
	StrHtmlCollectionOtherSpouses = StrHtmlCollectionMFU(collSpouses, "OtherSpouse", StrHtmlNarrativeNamesFull(collSpouses))
End Function
Function StrHtmlCollectionChildren(collChildren)
	StrHtmlCollectionChildren = StrHtmlCollectionMFU(collChildren , "Child", StrHtmlNarrativeNamesShort(collChildren))	
End Function
Function StrHtmlCollectionOtherChildren(collChildren)
	StrHtmlCollectionOtherChildren = StrHtmlCollectionMFU(collChildren , "ChildOther", StrHtmlNarrativeNamesShort(collChildren))	
End Function
Function StrHtmlCollectionStepChildren(collChildren)
	StrHtmlCollectionStepChildren = StrHtmlCollectionMFU(collChildren , "ChildStep", StrHtmlNarrativeNamesShort(collChildren))	
End Function
Function StrHtmlCollectionChildrenLocal(collChildren, strType)
	StrHtmlCollectionChildrenLocal = StrHtmlCollectionMFU(collChildren , "Child" & strType, StrHtmlNarrativeNamesShortLocal(collChildren))	
End Function

Function StrHtmlNarrativeNamesFull(coll)
	StrHtmlNarrativeNamesFull = StrHtmlNarrativeNames(coll, "NameFull", "", "")
End Function
Function StrHtmlNarrativeNamesShort(coll)
	StrHtmlNarrativeNamesShort = StrHtmlNarrativeNames(coll, "NameShort", "", "")
End Function
Function StrHtmlNarrativeNamesShortLocal(coll)
	StrHtmlNarrativeNamesShortLocal = StrHtmlNarrativeNames(coll, "NameShort", "#", "onclick='javascript:explorerTreeOpen(""Children"",""2"");'")
End Function

Function StrHtmlNarrativeName(i, strNameForm, strHrefPrefix, strAttribs)
	Dim strName, strTitle
	Dim strLabel
	fPrivateFolder = Eval(Left(Report.FileOutput,g_nPrivateFolder)=g_strPrivateFolder)
	' Fragment identifiers when a base tag is used seem to be unreliable
	If fPrivateFolder And strHrefPrefix = "#" Then
		strPrefix = g_strBaseurl & Report.FileOutput & "#"
		strLabel=Right(i.Href,12)
	Else
		strPrefix = strHrefPrefix
		strLabel = i.Href
	End If
	strName = i.Session(strNameForm)
	strTitle = ""
	If (strName <> "") Then
		strNameFull = i.Session("NameFull")
		If (strName <> strNameFull) Then
			' The name is different from the full name
				strTitle = Util.FormatString(" title='{&t}'", strNameFull)
		End If
		StrHtmlNarrativeName = Util.FormatString("<a href='{0}{1}' {2}{3}>{4&t}</a>", strPrefix, strLabel, strAttribs, strTitle, strName)
		If isPrivate(i) Then StrHtmlNarrativeName = Replace(StrHtmlNarrativeName, "<a", "<a class=""private""")
	Else
		StrHtmlNarrativeName = Util.HtmlEncode(StrDicMFU("_NoName",i.Gender.ID))
	End If
End Function

' Return a string containing the names of the collection of individuals.
' This string contains HTML tags, so it must be used with Report.Write() or with the WritePhrase's argument "{0h}".
' The implementation uses the Report buffer as a temporary storage, and extract the string from that buffer.
Function StrHtmlNarrativeNames(coll, strNameForm, strHrefPrefix, strAttribs)
	Dim i, iElement, iElementLast
	iElementLast = coll.Count- 1	' Get the index of the last element in the collection	
	
	Dim cchBuffer	' Number of characters in the buffer stream before writing the collection
	cchBuffer = Report.BufferLength
	
	Dim fValidName	' At least one name was valid in the collection
	Dim strSep		' Separator between the elements
	Dim strName, strNameFull, strTitle
	For iElement = 0 To iElementLast
		Set i = coll(iElement)
		Report.Write strSep
		If (i.Href <> "") Then
			fValidName = True
			Report.Write StrHtmlNarrativeName(i, strNameForm, strHrefPrefix, strAttribs)
		Else
			Report.WriteText StrDicMFU("_NoName",i.Gender.ID)
		End If
		
		If (iElement < iElementLast - 1) Then
			strSep = ", "
		Else
			strSep = Dic("ConjunctionAnd")
		End If
	Next

	If (fValidName) Then
		' Get the text from the Report buffer from where we started appending
		StrHtmlNarrativeNames = Report.Buffer(cchBuffer)
	End If
	Report.BufferLength = cchBuffer		' Truncate the buffer to its original size
End Function


' Write the individual for the Table of Contents
' The text contains HTML hyperlinks to the individual's report page
Sub WriteHtmlTocIndividual(i, hyperlink)
	Dim strFmtTemplate
	If hyperlink Then
		Report.WriteFormatted "<a href='{}' onclick='javascript:hidePopUpFrame();'>{&t}</a>", i.Href, i.Session("NameAlternative")
	Else
		Report.WriteFormatted "{&t}", i.Session("NameAlternative")
	End If
	Report.WritePhraseDic "FmtBirth", i.Birth.Date.Approximation, i.Birth.Date.Year, Dic("PhBC_" & i.Birth.CeremonyType.ID), i.Birth.Baptism.Date.Approximation, i.Birth.Baptism.Date.Year
End Sub

Sub WriteHtmlTocIndividualImgGender(i)
	Report.Write StrHtmlImgGender(i)
	WriteHtmlTocIndividual i, True
End Sub

Sub WriteHtmlTocIndividualContact(fTreeOpen, i, fPicturesOnly, fContactsOnly)
	Dim p
	If (Not fPicturesOnly And Not fContactsOnly) Or _
	    	(fPicturesOnly And Not Util.IsNothing(i.Pictures.Primary)) Or _
		(fContactsOnly And i.Contacts.Count > 0) Then
		If Not fContactsOnly Then
			WriteHtmlTocIndividualImgGender i
			If Not fPicturesOnly Then
				Report.WriteBr StrHtmlImgPhoto(i)
			Else
				Report.WriteBr
				For Each p in i.Pictures
					Report.WriteBr StrHtmlImgPhoto(p)
				Next
			End If
		Else
			If g_fUseTreeIndexes Then Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"")'>", Util.IfElse(fTreeOpen,"o","c")
			Report.Write StrHtmlImgGender(i)
			WriteHtmlTocIndividual i, False
			strFmtTemplate = "<a href='" & g_strPrivateFolder & "contacts.htm#{&t}' onclick='javascript:hidePopUpFrame();'><img src='images/{}.gif' alt='{}'/> {&t}</a> <small> ({})</small>"
			If g_fUseTreeIndexes Then
				Report.WriteLn "<ul>"
				strFmtTemplate = "<li class='xT-b'>" & strFmtTemplate & "</li>"
			Else
				Report.WriteBr
			End If
			For Each c In i.Contacts
				Report.WriteFormatted strFmtTemplate, c.ID, "occupancy", Dic("Occupancy"), Util.FirstNonEmpty(c.Place.Name, c.Summary, Util.IfElse(c.Comment<> "" And Len(c.Comment) < 80,c.Comment,"") ,Dic("Occupancy") & " (" & Dic("Ref") & c.ID & ")"), c.References.Count
			Next
			If g_fUseTreeIndexes Then Report.WriteLn "</ul></li>"
		End If
	End If
End Sub


' Write the HTML code to generate the Table of Contents for the indidiuals
' The parameter fPictureOnly is to exclude any individual without a picture
Sub WriteHtmlTocIndividuals(fTreeOpen, fPicturesOnly, fContactsOnly)

	Dim strFmtTemplate, strFmtTemplate1, strNameLast, cchBufferNameLast, cchBufferNames
	strFmtTemplate = "<a name='{0&t}'></a><span class='xT-i boldu'>{0&t}</span>"
	strFmtTemplate1 = strFmtTemplate
	If Not fPicturesOnly And Not fContactsOnly Then
		strFmtTemplate = strFmtTemplate & "&nbsp; <small>({&t})</small>"	' Include the count of individuals
	End If
	strFmtTemplate = strFmtTemplate & VbCrLf
	
	Dim oRepertoryIndividuals, o, oRepertoryFamilies, oStringDictionaryNames, oFamily, i, iCount
	iCount = Session("IndividualsCount")
	Set oRepertoryIndividuals = Session("oRepertoryIndividuals")
	Set oStringDictionaryNames = Session("oStringDictionaryNames")
	If (Not Util.IsNothing(oRepertoryIndividuals)) Then
		If g_fUseTreeIndexes Then
			strFmtTemplate = Util.FormatString("<li class='xT2-{}' onclick='xTclk(event,""2"")'>&nbsp;", Util.IfElse(fTreeOpen,"o","c")) & strFmtTemplate & "<ul>"

		Else
			strFmtTemplate = strFmtTemplate & "<br />"
		End If
	
		For Each o In oRepertoryIndividuals
			strFirstChar = o.Key
			If g_fUseTreeIndexes Then
				cchBufferLetter =  Report.BufferLength
				Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"");'>&nbsp;<span class='xT-i bold'>{&t}</span><ul>", Util.IfElse(fTreeOpen,"o","c"), strFirstChar		
				cchBufferFamilies =  Report.BufferLength
			End If
			Set oRepertoryFamilies = o.Object(0)
			For Each oFamily In oRepertoryFamilies 
				cchBufferNameLast = Report.BufferLength		' Remember the position where the last name was written
				strNameLast = oFamily.Key
				Report.WriteFormatted strFmtTemplate, strNameLast & oStringDictionaryNames.KeyValue(strNameLast), Dic.PlurialCount("Individual", oFamily.Count)
				cchBufferNames = Report.BufferLength
				fPrivateFolder = Eval(Left(Report.FileOutput,g_nPrivateFolder)=g_strPrivateFolder)
				For Each i In oFamily
					If (Not isPrivate(i) Or fPrivateFolder) Then WriteHtmlTocIndividualContact fTreeOpen, i, fPicturesOnly, fContactsOnly
				Next
				If (Report.BufferLength = cchBufferNames) Then
					' No name have been written, so flush the last name from the output stream.  This situation may happens when we want only individuals with pictures
					Report.BufferLength = cchBufferNameLast
				Else
					If g_fUseTreeIndexes Then
						Report.WriteLn "</li></ul>"
					Else
						Report.WriteBr
					End If
				End If
			Next
			If g_fUseTreeIndexes Then
				If Report.BufferLength = cchBufferFamilies Then
					' No families written with this initial letter so flush then letter header from the output stream
					Report.BufferLength = cchBufferLetter
				Else
					Report.WriteLn "</ul></li>"
				End If
			End If
		Next
	
		Set oRepertoryNoLastName = Session("oRepertoryNoLastName")
		If (oRepertoryNoLastName.Count > 0) Then
			cchBufferNameLast = Report.BufferLength
			Report.WriteFormattedBr Replace(strFmtTemplate,strFmtTemplate1,"<b>{0&t}</b>"),  Dic("_NoName"), Dic.PlurialCount("Individual", oRepertoryNoLastName.Count)
			cchBufferNames = Report.BufferLength
			For Each o In oRepertoryNoLastName
				For iCount = 0 To o.Count-1
					Set i = o.Object(iCount)
					If (Not isPrivate(i) Or fPrivateFolder) Then WriteHtmlTocIndividualContact fTreeOpen, i, fPicturesOnly, fContactsOnly
				Next
			Next
			If (Report.BufferLength = cchBufferNames) Then
				Report.BufferLength = cchBufferNameLast
			Else
				If g_fUseTreeIndexes Then Report.WriteLn "</ul></li>"
			End If			
		End If
	End If
End Sub

Sub WriteHtmlTocFamilies(fTreeOpen, fPicturesOnly)

	Dim collFamiliesSorted, f, strNameParent, strNameParentPrev, strFirstChar, strFirstCharPrev, strName, strPrefix
	Set collFamiliesSorted = Session("collFamiliesSorted")
	
	If (Not Util.IsNothing(collFamiliesSorted)) Then
		Dim cchBufferStart	' Start of the write operations
		Dim cchBufferValid	' Previous valid buffer.  If a sequence is not valid, flush the useless family name
		Dim cchBufferBegin	' Where is the begining of a valid sequence
		Dim cchBufferLetter	' Start of letter group
		Dim cchBufferFamliies	' Start of Family group
		cchBufferStart = Report.BufferLength
		cchBufferValid = cchBufferStart	' What was already written to the buffer is obviously valid
		cchBufferBegin = cchBufferStart

		strFmtTemplateName = "{}<a href='{}' onclick='javascript:hidePopUpFrame();'>{&t}</a>{}<br />"
		strFmtTemplateFamily = "<span class='xT-i boldu'>{&t}</span>"
		strFmtTemplateFamily1 = strFmtTemplateFamily


		If g_fUseTreeIndexes Then
			strFmtTemplateFamily = Util.FormatString("<li class='xT2-{}' onclick='xTclk(event,""2"");'>", Util.IfElse(fTreeOpen,"o","c")) & strFmtTemplateFamily & "<ul>"
		Else
			strFmtTemplateFamily = strFmtTemplateFamily & "<br />"
		End If

		fPrivateFolder = Eval(Left(Report.FileOutput,g_nPrivateFolder)=g_strPrivateFolder)
		For Each f In collFamiliesSorted
			If fPrivateFolder Or Not isPrivate(f) Then
			Set pp = f.Pictures.Primary
			If Not fPicturesOnly Or Not Util.IsNothing(pp) Then

				strNameParent = f.Parents(0).Session("NameLast")
				If strNameParent = "" Then strNameParent = f.Parents(1).Session("NameLast")
				strFirstChar = Util.StrStripAccentsUCase(Util.StrStripPunctuation(Util.StrGetFirstChar(strNameParent)))
				If strNameParent = "" Then
					strFirstChar = Dic("_NoName")
				End If
				If g_fUseTreeIndexes And (strFirstChar <> strFirstCharPrev) Then
					' We have a different initial letter
					If strFirstCharPrev <> "" Then
							Report.WriteLn "</ul></li></ul></li>"
					End If
					Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"");'> <span class='xT-i bold'>{&t}</span><ul>", Util.IfElse(fTreeOpen,"o","c"), strFirstChar
					strNameParentPrev = ""
				End If
				strFirstCharPrev = strFirstChar
					
				If (strNameParent <> strNameParentPrev) Then
					' We have a different family name than previous family name
					If strNameParentPrev <> "" Then
						If g_fUseTreeIndexes Then
							Report.WriteLn "</ul></li>"
						Else
							Report.WriteBr
							End If
					End If
					strNameParentPrev = strNameParent
					If StrNameParent <> "" Then Report.WriteFormatted strFmtTemplateFamily, strNameParent
				End If
				If Not fPicturesOnly Then
					Report.WriteFormattedLn strFmtTemplateName, StrHtmlImgFamily(f), f.Href, f.Session("Name"), StrHtmlImgPhoto(f)
				ElseIf Not Util.IsNothing(pp) Then
					Report.WriteFormattedLn strFmtTemplateName, StrHtmlImgFamily(f), f.Href, f.Session("Name"), ""
					For Each p in f.Pictures
						Report.WriteBr StrHtmlImgPhoto(p)
					Next
				End If
			End If
			End If
		Next
		If	strNameParentPrev <> "" And _
				strNameParent <> Dic("_NoName") Then
			If g_fUseTreeIndexes Then
				Report.Write "</ul></li>"
			Else
				Report.WriteBr
			End If
		End If
		If strFirstCharPrev <> "" Then
			If g_fUseTreeIndexes Then Report.WriteLn "</ul></li>"
		End If

	End If
	
End Sub
Sub WriteHtmlTocEducations(fTreeOpen, fPicturesOnly)
	Dim strFirstChar, strFirstCharPrev, strName, strNamePrev, e, pic

	Dim oDataSorter, collEducations

	Set oDataSorter = Util.NewDataSorter()
	For Each e in Educations
		oDataSorter.Add e, Util.FirstNonEmpty(e.Institution, e.Place.Session("NameFull"))
	Next

	oDataSorter.SortByKey
	Set collEducations = oDataSorter.ToGenoCollection

	If g_fUseTreeIndexes Then
		For Each e in collEducations
			strName = Util.FirstNonEmpty(e.Institution, e.Place.Session("NameFull"))
			If Not fPicturesOnly or e.Pictures.Count > 0 Then
				strFirstChar = Util.StrStripAccentsUCase(Util.StrStripPunctuation(Util.StrGetFirstChar(strName)))
				if strFirstChar = "" Then strFirstChar = " "
				if strFirstChar <> strFirstCharPrev Then
					If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
					Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"");'> <span class='xT-i bold'>{&t}</span><ul>", Util.IfElse(fTreeOpen,"o","c"), strFirstChar
					strFirstCharPrev = strFirstChar
				End If

				Report.WriteformattedLn "<li class='xT-b'><img src='images/education.gif' class='icon' />&nbsp;{}<small> ({})</small></li><ul>", strName, e.References.Count
				For Each d in e.References
					Report.Write StrHtmlImgGender(d)
					Report.WriteFormattedBr"<a href='{}#{}' onclick='javascript:hidePopUpFrame();'>{&t}</a>", d.Href, e.ID, d.Name
				Next
				If fPicturesOnly Then
					For Each pic in e.Pictures
						Report.WriteBr StrHtmlImgPhoto(pic)
					Next
				End If
				Report.WriteLn "</ul>"
			End If
		Next
		If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
	End If
End Sub

Sub WriteHtmlTocRelationships(Relationships, fTreeOpen)
	Dim strFirstChar, strFirstCharPrev, strName, strNamePrev, o, pic, strComments

	If g_fUseTreeIndexes Then
		For Each r in Relationships
			If r.Pictures.Count > 0 Then
				If r.Class = "EmotionalRelationship" Then
					strFirstChar = Util.StrStripAccentsUCase(Util.StrStripPunctuation(Util.StrGetFirstChar(r.EmotionalLink.ID)))
				Else
					strFirstChar = Util.StrStripAccentsUCase(Util.StrStripPunctuation(Util.StrGetFirstChar(r.Connection.ID)))
				End If
				if strFirstChar = "" Then strFirstChar = " "
				if strFirstChar <> strFirstCharPrev Then
					If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
						Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"");'> <span class='xT-i bold'>{&t}</span><ul>", Util.IfElse(fTreeOpen,"o","c"), strFirstChar
					strFirstCharPrev = strFirstChar
				End If
				Select Case r.Class
				Case "EmotionalRelationship"
					Report.WriteformattedLn "<li class='xT-b'><img src='images/yinyan.gif' class='icon' />&nbsp;{}</li><ul>", r.EmotionalLink
					Report.Write StrHtmlImgGender(r.Entity1)
					Report.WriteFormattedLn"<a href='{}' onclick='javascript:hidePopUpFrame();'>{&t}</a>,&nbsp;", r.Entity1.Href, Util.FirstNonEmpty(r.Entity1.Name, Dic("_NoName"))
					Report.Write StrHtmlImgGender(r.Entity2)
					Report.WriteFormattedBr"<a href='{}' onclick='javascript:hidePopUpFrame();'>{&t}</a>", r.Entity2.Href, Util.FirstNonEmpty(r.Entity2.Name, Dic("_NoName"))
				Case Else
					Report.WriteformattedLn "<li class='xT-b'><img src='images/yinyan.gif' class='icon' />&nbsp;{&t}</li><ul>", Util.FirstNonEmpty(r.Connection, Dic("_NoName"))
					WriteHtmlRelationshipEntity r.entity1, r
					Report.Write ",&nbsp;"
					WriteHtmlRelationshipEntity r.entity2, r
					Report.WriteBr
				End Select
				For Each pic in r.Pictures
					Report.WriteBr StrHtmlImgPhoto(pic)
				Next
				Report.WriteLn "</ul>"
			End If
		Next
		If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
	End If
End Sub

Sub WriteHtmlRelationshipEntity(e, r)
	Select Case e.Class
	Case "Individual"
		Report.Write StrHtmlImgGender(e)
		If e.Name <> "" Then
			Report.WriteFormatted "<a href='{}' onclick='javascript:hidePopUpFrame();'>{&t}</a>", e.Href, e.Name
		Else
			Report.WriteText Dic("_NoName")
		End If
	Case "SocialEntity"
		Report.Write Replace(e.Text, vbLf, " ")
	End Select
End Sub

Sub WriteHtmlTocOccupations(fTreeOpen, fPicturesOnly)
	Dim strFirstChar, strFirstCharPrev, strName, strNamePrev, o, pic

	Occupations.SortBy("Place Company")

	If g_fUseTreeIndexes Then
		For Each o in Occupations
			If Not fPicturesOnly or o.Pictures.Count > 0 Then
				strFirstChar = Util.StrStripAccentsUCase(Util.StrStripPunctuation(Util.StrGetFirstChar(o.Place.Name)))
				if strFirstChar = "" Then strFirstChar = " "
				if strFirstChar <> strFirstCharPrev Then
					If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
					Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"");'> <span class='xT-i bold'>{&t}</span><ul>", Util.IfElse(fTreeOpen,"o","c"), strFirstChar
					strFirstCharPrev = strFirstChar
				End If

				Report.WriteformattedLn "<li class='xT-b'><img src='images/occupation.gif' class='icon' />&nbsp;{}<small> ({})</small></li><ul>", o.Place & Util.IfElse(o.Company <> "", ", " & o.Company,""), o.References.Count
				For Each d in o.References
					Report.Write StrHtmlImgGender(d)
					Report.WriteFormattedBr"<a href='{}#{}' onclick='javascript:hidePopUpFrame();'>{&t}</a>", d.Href, o.ID, d.Name
				Next
				If fPicturesOnly Then
					For Each pic in o.Pictures
						Report.WriteBr StrHtmlImgPhoto(pic)
					Next
				End If
				Report.WriteLn "</ul>"
			End If
		Next
		If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
	End If
End Sub

Sub WriteHtmlTocPlaces(fTreeOpen, fPicturesOnly)
	Dim strFirstChar, strFirstCharPrev, strName, strNamePrev, p, oDataSorter, collPlaces

	Set oDataSorter = Util.NewDataSorter()
	For Each p in Places
		oDataSorter.Add p, p.Session("NameFull")
	Next

	oDataSorter.SortByKey
	Set collPlaces = oDataSorter.ToGenoCollection

	If g_fUseTreeIndexes Then
		For Each p in collPlaces
			If Not fPicturesOnly or p.Pictures.Count > 0 Then
				strFirstChar = Util.StrStripAccentsUCase(Util.StrStripPunctuation(Util.StrGetFirstChar(p.Session("NameFull"))))
				if strFirstChar = "" Then strFirstChar = " "
				if strFirstChar <> strFirstCharPrev Then
					If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
					Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"");'> <span class='xT-i bold'>{&t}</span><ul>", Util.IfElse(fTreeOpen,"o","c"), strFirstChar
					strFirstCharPrev = strFirstChar
				End If

				AddPlaceNode p, 0, fTreeOpen, fPicturesOnly
			End If
		Next
		If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
	Else
		For Each p In collPlaces
			If Not fPicturesOnly or p.Pictures.Count > 0 Then
				strName = p.Session("NameFull")
				Report.WriteFormatted "<img src='images/place.gif' class='icon' alt='{}'/> <a href='places.htm#{&t}' title='{&t}' onclick='javascript:hidePopUpFrame();'>{&t}</a> <small> ({})</small>", Dic("Place"), p.ID, p.Category, strName, p.References.Count
				Report.WriteBr StrHtmlImgPhotoLink(p, "places.htm")
			End If
		Next
	End If
End Sub

Sub WriteHtmlTocSources(fTreeOpen, fPicturesOnly)
	Dim strFirstChar, strFirstCharPrev, strName, strNamePrev, s, collSourcesAndCitiations, oDataSorter

	Set oDataSorter = Util.NewDataSorter()

	' Add each source to the DataSorter
	For Each s In SourcesAndCitations
		strName=UCase(Util.StrStripAccentsUCase(Util.StrStripPunctuation(StrParseText(s.Title))))
		oDataSorter.Add s, strName, s.Subtitle, s.Description, s.WhereInSource
	Next

	' Sort the sources according the sort keys
	oDataSorter.SortByKey
	Set collSourcesAndCitations = oDataSorter.ToGenoCollection

	If g_fUseTreeIndexes Then
		For Each s in collSourcesAndCitations
			If Not fPicturesOnly or s.Pictures.Count > 0 Then
				strFirstChar = Util.StrGetFirstChar(Util.StrStripAccentsUCase(Util.StrStripPunctuation(StrParseText(s))))
				if strFirstChar = "" Then strFirstChar = " "
				if strFirstChar <> strFirstCharPrev Then
					If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
					Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"");'> <span class='xT-i bold'>{&t}</span><ul>", Util.IfElse(fTreeOpen,"o","c"), strFirstChar
					strFirstCharPrev = strFirstChar
				End If

				AddSourceCitationNode s, 0, fTreeOpen, fPicturesOnly
			End If
		Next
		If strFirstCharPrev <> "" Then 	Report.WriteLn "</ul></li>"
	Else
		For Each s In collSourcesAndCitations
			If Not fPicturesOnly or s.Pictures.Count > 0 Then
				strName = JoinSourceCitationNames(s, StrParseText(s.title), true)
				Report.WriteFormatted "<img src='images/source.gif' border='0' width='16' height='16' alt='{}'/> <a href='sources.htm#{&t}' title='{&t}' onclick='hidePopUpFrame();'>{&t}</a> <small> ({})</small>", Dic("Source"), s.ID, s.Subtitle, strName, s.References.Count
				Report.WriteBr StrHtmlImgPhotoLink(s, "sources.htm")
			End If
		Next
	End If
End Sub

Function AddPlaceNode(p, level, fTreeOpen, fPicturesOnly)
	If (level >= 20) Then
		Exit Function	' Maximum of 20 levels, to prevent an infinite loop
	End If
	Dim strHtmlPlace, pchild, pic, strName, strImgLink
'	strName = JoinPlaceNames(p, p.Name, level = 0)
	If level = 0 Then
		strName = p.Session("NameFull")
	Else
		strName = p.Session("NameShort")
	End If
	If Not fPicturesOnly Then strImgLink = StrHtmlImgPhotoLink(p, "places.htm")
	strHtmlPlace = Util.FormatString("<img src='images/place.gif' class='icon' />&nbsp;<a href='places.htm#{&t}' title='{&t}' onclick='javascript:hidePopUpFrame();'>{&t}</a>" & strImgLink, p.ID, p.Category, strName)
	If Not Util.IsNothing(p.Children) Then
		Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"")'>" & strHtmlPlace, Util.IfElse(fTreeOpen,"o","c")
		Report.WriteFormattedLn " <small> ({})</small>", Dic.FormatPhrase("PhUsageCounters",p.References.Count, p.Children.Count)
		Report.WriteLn "<ul>"
		If fPicturesOnly Then
			For Each pic in p.Pictures
				Report.WriteBr "<img src='images/space.gif' width='16' alt=''/>" & StrHtmlImgPhoto(pic)
			Next
		End If
		For Each pchild in p.Children
			AddPlaceNode pchild, level+1, fTreeOpen, fPicturesOnly
		Next
		Report.WriteLn "</ul></li>"
	Else
		Report.WriteformattedLn "<li class='xT-b'>{} <small> ({})</small></li>", strHtmlPlace, p.References.Count
		If fPicturesOnly Then
			Report.WriteLn "<ul>"
			For Each pic in p.Pictures
				Report.WriteBr StrHtmlImgPhoto(pic)
			Next
			Report.WriteLn "</ul>"
		End If
	End If
End Function

Function JoinPlaceNames(p, strName, toplevel)
	Dim strT0, strT1, strT2, j
	If g_fJoinPlaceNames And Not Util.IsNothing(p.Parent) And toplevel Then
		strT0 = p.Session("NameShort")
		strT0 = strName
		strT1 = p.Parent.Session("NameShort")
		strT2 = JoinPlaceNames(p.Parent, strT1, False)
		If Right(strT0,1)="]" then strT0=left(strT0,Len(strT0)-1)
		If Right(strT0,Len(strT1)) <> strT1 And Right(strT0,Len(strT2)) <> strT2 Then _
		JoinPlaceNames = JoinPlaceNames(p.Parent, strName & ", " & p.Parent.Session("NameShort"), toplevel) 
	Else
		JoinPlaceNames = strName
	End If
End Function

Function JoinSourceCitationNames(s, strName, toplevel)
	Dim strNewName, fCitation
	strNewName = strName
	fCitation = false
	If  Not Util.IsNothing(s.Parent) Then
		If s.WhereInSource <> "" and strName = s.Parent.title Then
			strNewName = "'" & s.WhereInSource & "'"
			fCitation = True
		End If
		If g_fJoinSourceCitationNames And Not Util.IsNothing(s.Parent) And toplevel Then
			If fCitation Or (s.title <> s.Parent.title) Then strNewName = strNewName &  " @ " & StrParseText(s.Parent.title)
			JoinSourceCitationNames = JoinSourceCitationNames(s.Parent, strNewName, toplevel) 
		Else
			JoinSourceCitationNames = strNewName
		End If
	Else
		JoinSourceCitationNames = strNewName
	End If
End Function

Function AddSourceCitationNode(s, level, fTreeOpen, fPicturesOnly)
	If (level >= 20) Then
		Exit Function	' Maximum of 20 levels, to prevent an infinite loop
	End If
	Dim strHtmlSourceCitation, p, schild, strName, strImgLink
	strName = JoinSourceCitationNames(s, StrParseText(s.title), level = 0)
	If Not fPicturesOnly Then strImgLink = StrHtmlImgPhotoLink(s, "sources.htm")
	strHtmlSourceCitation = Util.FormatString("<img src='images/source.gif' class='icon' />&nbsp;<a href='sources.htm#{&t}' title='{&t}' onclick='javascript:hidePopUpFrame();'>{&t}</a>" & strImgLink, s.ID, s.MediaType, strName)
	strHtmlSourceCitation=Replace(Replace(strHtmlSourceCitation,"{","&#123;"),"}","&#125;")
	If Not Util.IsNothing(s.Children) Then
		Report.WriteFormattedLn "<li class='xT2-{}' onclick='xTclk(event,""2"")'>" & strHtmlSourceCitation, Util.IfElse(fTreeOpen,"o","c")
		Report.WriteFormattedLn " <small> ({})</small>", Dic.FormatPhrase("PhUsageCounters",s.References.Count, s.Children.Count)
		Report.WriteLn "<ul>"
		If fPicturesOnly Then
			For Each p in s.Pictures
				Report.WriteBr StrHtmlImgPhoto(p)
			Next
		End If
		For Each schild in s.Children
			If Not fPicturesOnly Or schild.Pictures.Count>0 Then
				AddSourceCitationNode schild, level+1, fTreeOpen, fPicturesOnly
			End If
		Next
		Report.WriteLn "</ul></li>"
	Else
		Report.WriteformattedLn "<li class='xT-b'>{} <small> ({})</small></li>", strHtmlSourceCitation, s.References.Count
		If fPicturesOnly Then
			For Each p in s.Pictures
				Report.WriteLn "<ul>"
				Report.WriteBr StrHtmlImgPhoto(p)
				Report.WriteLn "</ul>"
			Next
		End If
	End If
End Function

Sub WriteHtmlSource(s)
	Dim collReferences, pub, ser, strClassCitation
	Set pub = s.Publication
	Set ser = s.Series
	strClassCitation="citation"
	Report.WriteFormattedLn "<a name='{&t}'></a><h3>{&t}</h3>", s.ID, JoinSourceCitationNames(s, StrParseText(s.title), true)

	If (s.Pictures.Count > 0) Then
		Report.WriteLn "<div class='floatright aligncenter widthpaddedlarge'>"
		WriteHtmlPicturesLarge s, "left", "", g_fHidePictureDetails
		Report.WriteLn "</div>"
		strClassCitation="citationpic"
	End If
	
	If s.subtitle <> "" Then Report.WriteFormattedLn "<div class='subhead'>{}</div>", StrFormatText(s, s.subtitle)
	If s.QuotedText <> "" Then Report.WriteFormattedLn "<div class='{0}'>&ldquo;{1}&rdquo;</div>", strClassCitation, StrFormatText(s, s.QuotedText)
	Report.WritePhraseDic "PhSourceEntry", s.Edition, ser.Name, ser.Issue, pub.publisher, pub.Place.Session("HlinkLocative"), _
				pub.Date.ToStringNarrative, s.Originator , StrFormatText(s, s.Description), s.WhereInSource, s.Editor, _
				s.ISBN, Util.FormatHtmlHyperlink(s.Url, s.Url), s.Repository.Session("HlinkLocative"), s.MediaType, s.ReferenceNumber, s.ConfidenceLevel
	
	WriteHtmlExtraNarrative s

	WriteHtmlAdditionalInformation(s)
	WriteHtmlAnnotation s, Dic("AnnotationSource"), s.Comment
	WriteHtmlReferences s, false
End Sub

Sub WriteHtmlPlace(p, fGoogleMaps, fLink)
	Dim strWidth, strHeight, strType, strZoom
	Set collReferences = p.References
	Report.WriteFormattedLn "<a name='{&t}'></a><h3>{&t}</h3>", p.ID, p.Session("NameFull")
	If (p.Pictures.Count > 0) Then
		Report.WriteLn "<div class='floatright aligncenter widthpaddedlarge'>"
		WriteHtmlPicturesLarge p, "left", "", g_fHidePictureDetails
		Report.WriteLn "</div>"
		If fGoogleMaps Then
			strWidth = CustomTag(p, "Map.Google.Width")
			If strWidth="" Then strWidth = g_cxGoogleMapsSmall
			strHeight= CustomTag(p, "Map.Google.Height")
			If strHeight="" Then strHeight = g_cyGoogleMapsSmall
		End If
	ElseIf fGoogleMaps Then
		strWidth = CustomTag(p, "Map.Google.Width")
		If strWidth="" Then strWidth = g_cxGoogleMapsLarge
		strHeight= CustomTag(p, "Map.Google.Height")
		If strHeight="" Then strHeight = g_cyGoogleMapsLarge
	End If
	Dim strZip
	If p.Zip <> "" And p.Country = "UK" Then
		strZip = Util.FormatString("<a href='http://www.Streetmap.co.uk/streetmap.dll?postcode2map?{0}&{1&t}' target=_blank>{2}</a>",Replace(p.Zip, " ","+"), Replace(p.Name, " ","+"), p.Zip)
	Else
		strZip = p.Zip
	End If
	Report.WritePhraseDic "PhPlaceDescription", p.Session("NameShort"), Util.IfElse(p.Category <> p.Parent.Category, p.Category, ""), p.Street, p.Session("City"), p.Session("County"), p.Session("State"), strZip, p.Session("Country"), _
					"", "", "", p.Latitude, p.Longitude, "<b>", "</b>"
	Report.Write3Br "", Util.IfElse(p.Description <> p.Parent.Description, StrFormatText(p, p.Description), ""), "" 
	WriteHtmlExtraNarrative p

	If fGoogleMaps Then
		If fLink Then ' link to map
			strPlace=Util.FormatPhrase("{0}[[{?0},]{1}][[{?0|1},]{2}][[{?0|1|2},]{3}][[{?0|1|2|3},]{4}][[{?0|1|2|3|4},]{5}]",p.Street, p.Session("City"), p.Session("County"), p.Session("State"), p.Zip, p.Session("Country"))
			if strPlace = "" Then strPlace=p.Session("NameFull")
			strZoom=CustomTag(p, "Map.Google.Zoom")
			if strZoom = "" Then strZoom = g_strGoogleMapsZoom
			strType=CustomTag(p, "Map.Google.Type")
			if strType = "" Then strType = g_strGoogleMapsType
			Report.WriteFormattedLn "<a href='javascript:checkGoogleMap(""{&j}"", ""{&j}"", ""{&j}"", {}, {},document.location,false);'>{&t}</a>",p.Latitude, p.Longitude, strPlace, strType, strZoom, Dic("gMapLink")
		Else ' show map on this page
			Report.WriteLn		"<div id='GoogleMapWrapper'><ul class='xT'>"
			Report.WriteFormattedLn "    <li class='xT3-o xT-h' onclick='xTclk(event,""3"")'>{&t}", Dic("GoogleMap")
			Report.WriteLn 		"        <ul class='xT-b'>"
			Report.WriteFormattedLn	"		<li><div id='GoogleMap' style='width: {}px; height: {}px;'></div></li>", strWidth, strHeight
			Report.WriteLn 		"        </ul>"
			Report.WriteLn		"    </li>"
			Report.WriteLn		"</ul></div>"
		End If
	ElseIf g_fGoogleMaps Then ' link to map (if available) in popup frame
			strPlace=Util.FormatPhrase("{0}[[{?0},]{1}][[{?0|1},]{2}][[{?0|1|2},]{3}][[{?0|1|2|3},]{4}][[{?0|1|2|3|4},]{5}]",p.Street, p.Session("City"), p.Session("County"), p.Session("State"), p.Zip, p.Session("Country"))
			if strPlace = "" Then strPlace=p.Session("NameFull")
			strZoom=CustomTag(p, "Map.Google.Zoom")
			if strZoom = "" Then strZoom = g_strGoogleMapsZoom
			strType=CustomTag(p, "Map.Google.Type")
			if strType = "" Then strType = g_strGoogleMapsType
			Report.WriteFormattedLn "<a href='javascript:checkGoogleMap(""{&j}"", ""{&j}"", ""{&j}"", {}, {},"""",false);'>{&t}</a>",p.Latitude, p.Longitude, strPlace, strType, strZoom, Dic("gMapLink")
	End If
	WriteHtmlAdditionalInformation(p)
	WriteHtmlAnnotation p, Dic("AnnotationLocation"), p.Comment
	WriteHtmlReferences p, false
	WriteHtmlAllFootnotes p.Sources

End Sub

Sub WriteHtmlReferences(o, nested)
	Dim collReferences, oParent, d, oDate, oDateYears
	Set oParent = Nothing
	Set collReferences = o.References
	Set oDateYears=Util.NewStringDictionary
	Select Case o.Class
	Case  "SourceCitation"
		Set oParent = o.Parent
	Case  "Place"
		Set oParent = o.Parent
		Set collReferences = Util.NewDataSorter
		For Each r in o.References
			Set oDate = Nothing
			Select Case r.Class
			Case "Individual"
				Set oDate = MatchDate(o, r.Birth.Place, r.Birth.Date)
				If Util.IsNothing(oDate) Then Set oDate = MatchDate(o, r.Birth.Baptism.Place, r.Birth.Baptism.Date)
				If Util.IsNothing(oDate) Then Set oDate = MatchDate( o, r.Death.Place, r.Death.Date)
				If Util.IsNothing(oDate) Then Set oDate = MatchDate(o, r.Death.Funerals.Place, r.Death.Funerals.Date)
				If Util.IsNothing(oDate) Then Set oDate = MatchDate(o, r.Death.Disposition.Place, r.Death.Disposition.Date)
			Case "Education", "Occupation", "Contact"
				If Not r.DateStart.ToStringNarrative = "" Then
					Set oDate = r.DateStart
				Else
					Set oDate = r.DateEnd
				End If
			Case "Marriage"
				Set oDate = MatchDate(o, r.Place, r.Date)
				If Util.IsNothing(oDate) Then Set oDate = MatchDate(o, r.Divorce.Place, r.Divorce.Date)
			End Select
			collReferences.Add r, GetDate(oDate), r.Class
			If Not Util.IsNothing(oDate) Then oDateYears.Add r.Class & r.ID, oDate.Year
		Next
		collReferences.SortByKey()
		Set collReferences = collReferences.ToGenoCollection
	End Select
	If collReferences.Count > 0 Or Not Util.IsNothing(oParent) Then
		Report.WriteFormattedLn	"<ul class='xT {}'>",Util.IfElse(nested,"","note")	' bug in IE has wrong interpretationof 'smaller; font
		Report.WriteFormattedLn "    <li class='xT2-{} xT-h' onclick='xTclk(event,""2"")'>{&t} ({})", Util.IfElse(g_fCollapseReferences, "c", "o"), Dic.FormatPhrase("PhReferences", Dic(o.Class)), collReferences.Count + Util.IfElse(Util.IsNothing(oParent),0,1)
		Report.WriteLn 		"        <ul class='xT-n'>"
		If Not Util.IsNothing(oParent) Then
			g_ichReferencesStart = -1
			Report.WriteFormattedLn "<li>{&t}: {}</li>", Dic(o.Class & "_Parent"), oParent.Session("Hlink")
		End If
		For Each d In collReferences
			g_ichReferencesStart = -1
			strHref = d.Href
			If (strHref <> "") Then
				Select Case d.Class
					Case "Place"
						strHref = d.Session("Hlink")
 					Case "SourceCitation"
						strHref = d.Session("Hlink")
					Case "Family"
						strHref = Util.FormatString("<a href='{&t}' target='detail' onclick='javascript:hidePopUpFrame();'>{&t}</a>", strHref, d.Session("Name"))
					Case "Individual"
						strHref = d.Session("Hlink")
						If o.Class = "Place" Then
							  strHref = strHref & Util.FormatPhrase(Dic("FmtDatesIndividual"), _
								Util.IfElse(o.ID = d.Birth.Place.ID, GetDate(d.Birth.Date),""), _
								Dic("PhBC_" & d.Birth.CeremonyType.ID), _
								Util.IfElse(o.ID = d.Birth.Baptism.Place.ID, GetDate(d.Birth.Baptism.Date),""), _
								Util.IfElse(o.ID = d.Death.Place.ID, GetDate(d.Death.Date),""), _
								Util.IfElse(o.ID = d.Death.Funerals.Place.ID, GetDate(d.Death.Funerals.Date),""), _
								Util.IfElse(o.ID = d.Death.Disposition.Place.ID, GetDate(d.Death.Disposition.Date),""))
						End If
				End Select	
			Else
				strSep = ""	
				Select Case d.Class
					Case "Occupation", "Education"
						strHref = StrParseText(d) & " "
						For Each dobj in d.References
							strHref = strHref & strSep & Util.FormatString("<a href='{&t}' target='detail' onclick='javascript:hidePopUpFrame();'>{&t}</a>", dobj.Href, dobj)
							strSep = ", "
						Next
						If o.Class = "Place" And o.ID = d.Place.ID Then
							strHref = strHref & Util.FormatPhrase(Dic("FmtDatesFromTo"), _
								GetDate(d.DateStart), _
								GetDate(d.DateEnd))
						End If
					Case "PedigreeLink"
						strHref = d.PedigreeLink & " " & strSep & Util.FormatString("<a href='{&t}' target='detail' onclick='javascript:hidePopUpFrame();'>{&t}</a>", d.individual.Href, d.individual)
					Case "Marriage", "Contact"
						If d.Class = "Contact" And d.Type.ID <> "" Then strHref = d.Type & " "
						For Each dobj in d.References
							strHref = strHref & strSep & Util.FormatString("<a href='{&t}' target='detail' onclick='javascript:hidePopUpFrame();'>{&t}</a>", dobj.Href, dobj)
							strSep = ", "
						Next
						If o.Class = "Place" Then
							If d.Class = "Contact" And o.Id = d.Place.ID Then
								strHref = strHref & Util.FormatPhrase(Dic("FmtDatesFromTo"), _
									GetDate(d.DateStart), _
									GetDate(d.DateEnd))
							Else
								strHref = strHref & Util.FormatPhrase(Dic("FmtDatesUnion"), _
									Util.IfElse(o.ID = d.Place.ID, GetDate(d.Date),""), _
									Util.IfElse(o.ID = d.Divorce.Place.ID, GetDate(d.Divorce.Date),""))
							End If
						End If
					Case Individual
						strHref = d.Session("Hlink")
					Case Else
						strHref = d & ""
				End Select
			End If
			Select Case d.Class
			Case "SocialRelationship", "EmotionalRelationship"
				Report.WriteFormatted "<li>{&t}: ", Dic(d.Class)
				WriteHtmlRelationship d, "", "", "", "", False, False
				Report.WriteLn "</li>"
			Case Else
				Report.WriteFormattedLn "<li>{} {&t}: {}</li>", oDateYears.KeyValue(d.Class & d.ID), Dic(d.Class), strHref
			End Select
		Next
		Report.WriteLn "        </ul>"
		Report.WriteLn "    </li>"
		Report.WriteLn "</ul>"
	End If
End Sub

Function WriteHtmlRelationship(r, strName1,strNameObj1, strName2, strNameObj2, fSwitch, fSources)
	Dim strHref, strHref2, strTitle, strRelationship, relation,  fExtant, strGender1, strGender2, strRef1, strRef2, strObj1, strObj2
	strHref2 = ""
	Select Case r.Class
	Case "EmotionalRelationship"
		Set e1 = HyperlinkDataSource(r.Entity1)
		Set e2 = HyperlinkDataSource(r.Entity2)
		strRelationship = Dic("PhER_" & r.EmotionalLink.ID)
	Case "SocialRelationship"
		Set e1 = HyperlinkDataSource(r.entity1)
		Set e2 = HyperlinkDataSource(r.entity2)
		strRelationship = Dic("PhSR_" & r.Connection.ID)
	End Select

	Select Case e1.Class
	Case "Individual"
		fExtant = (Util.GetStatisticsForIndividuals(e1) = 1)
		strRef1 = Util.FirstNonEmpty(strName1, e1.Session("HlinkNN"))
		strObj1 = strNameObj1
		If strObj1 = "" Then strObj1 = strRef1
		strGender1 = e1.Gender.ID
	Case "SocialEntity"
		strRef1 = Replace(e1.Text,vbLf," ")
	Case Else
		strRef1 = e1 & "" ' use default property
	End Select

	Select Case e2.Class
	Case "Individual"
		fExtant = fExtant And (Util.GetStatisticsForIndividuals(e2) = 1)
		strRef2 = Util.FirstNonEmpty(strName2, e2.Session("HlinkNN"))
		strObj2 = strNameObj2
		If strObj2 = "" Then strObj2 = strRef2
		strGender2 = e2.Gender.ID
	Case "SocialEntity"
		strRef2 = Replace(e2.Text,vbLf," ")
	Case Else
		strRef2 = e2 & "" ' use default property
	End Select
	If Not fSwitch Then
		Report.WritePhrase strRelationship, Util.StrFirstCharUCase(strRef1), fExtant = True, strObj2, strGender1="F", strGender2="F", strGender1 = strGender2, strObj1, strRef2
	Else
		Report.WritePhrase strRelationship, Util.StrFirstCharUCase(strRef2), fExtant = True, strObj1, strGender2="F", strGender1="F", strGender1 = strGender2, strObj2, strRef1
	End If
	If fSources Then WriteHtmlFootnoteRefs(r.Sources)

End Function

Function HyperlinkDataSource(obj)
	If obj.Class = "Individual" Then
		If Not Util.IsNothing(obj.IndividualInternalHyperlink) Then
			Set HyperlinkDataSource = obj.IndividualInternalHyperlink
		Else
			Set HyperlinkDataSource = obj
		End If
	Else
		Set HyperlinkDataSource = obj
	End If
End Function

Sub WriteHtmlLinkToPrivateToc(fPrivate, strWhat)
	If fPrivate Then
		Report.WriteFormattedLn "<p><a href=""toc_{}.htm"" target=""_self"">", strWhat
		Report.WriteFormattedLn "Restore list of public {}.</a></p>", strWhat
	Else
		Report.WriteFormattedLn "<p>This is an index of just those {} whose details are public.", strWhat
		Report.WriteFormattedLn "<a class=""private"" href=""private/toc_{}.htm"" target=""_self"">", strWhat
		Report.WriteFormattedLn "See private {} (password protected).</a></p>", strWhat
	End If
End Sub




